@@ -5,10 +5,29 @@
 
 ######################### We start with some black magic to print on failure.
 
-# Change 1..1 below to 1..last_test_to_print .
-# (It may become useful if the test is moved to ./t subdirectory.)
+# Change 1..1 below to 1..last_test_to_print. (It may become useful if
+# the test is moved to ./t subdirectory.) Remember that all the tests
+# except the first are done twice--once with Storable, and once with
+# Data::Dumper. The $TEST_SET_SIZE is the number of unique tests, not
+# counting the trivial first test. @PERSISTENCE_MECHANISMS is an array
+# containing all the supported persistence mechanisms
+
+use vars qw($TEST_SET_SIZE @PERSISTENCE_MECHANISMS);
+
+BEGIN
+{
+  $| = 1;
+  $TEST_SET_SIZE = 25;
+  @PERSISTENCE_MECHANISMS = qw(Data::Dumper Storable);
+
+  # The test set is repeated once for each implementation, plus the
+  # first test
+  my $last_test_to_print = 
+    (($TEST_SET_SIZE) * ($#PERSISTENCE_MECHANISMS + 1)) + 1;
+
+  print "1..$last_test_to_print\n";
+}
 
-BEGIN { $| = 1; print "1..21\n"; }
 END {print "not ok 1\n" unless $loaded;}
 
 use File::Cache qw($sSUCCESS $sFAILURE);
@@ -17,7 +36,6 @@ use File::Cache qw($sSUCCESS $sFAILURE);
 $loaded = 1;
 print "ok 1\n";
 
-
 ######################### End of black magic.
 
 use strict;
@@ -28,9 +46,45 @@ my $sMAX_SIZE = 1000;
 my $sTEST_USERNAME = "web";
 my $sTEST_CACHE_DEPTH = 3;
 
+# Run all remaining tests for each implementation
+my $test_set_number = 0;
+
+foreach my $implementation (@PERSISTENCE_MECHANISMS)
+{
+  $test_set_number++;
+  my $test_set_start = $TEST_SET_SIZE * ($test_set_number - 1) + 2;
+  my $test_set_end = $TEST_SET_SIZE * $test_set_number + 1;
+
+  # Only do the tests if the persistence mechanism module is present
+  if (eval "require $implementation")
+  {
+    do_tests($implementation, $test_set_start);
+  }
+  else
+  {
+    skip_tests($test_set_start, $test_set_end);
+  }
+}
+
+sub skip_tests
+{
+  my ($start,$end) = @_;
+
+  for (my $i = $start; $i <= $end;$i++)
+  {
+    print "ok $i # skip\n";
+  }
+}
+
+sub do_tests
+{
+my ($implementation,$test_number_start) = @_;
+
+print "--> Testing $implementation implementation\n";
+
 # Test creation of a cache object
 
-my $test = 2;
+my $test = $test_number_start;
 
 my $cache1 = new File::Cache( { cache_key => $sTEST_CACHE_KEY,
 				namespace => $sTEST_NAMESPACE,
@@ -38,6 +92,7 @@ my $cache1 = new File::Cache( { cache_key => $sTEST_CACHE_KEY,
 			        auto_remove_stale => 0,
 			        username => $sTEST_USERNAME,
 			        filemode => 0770,
+				implementation => $implementation,
 				cache_depth => $sTEST_CACHE_DEPTH } );
 
 if ($cache1) {
@@ -48,7 +103,7 @@ if ($cache1) {
 
 # Test the setting of a scalar in the cache
 
-$test = 3;
+$test++;
 
 my $seed_value = "Hello World";
 
@@ -64,7 +119,7 @@ if ($status == $sSUCCESS) {
 
 # Test the getting of a scalar from the cache
 
-$test = 4;
+$test++;
 
 my $val1_retrieved = $cache1->get($key);
 
@@ -74,37 +129,71 @@ if ($val1_retrieved eq $seed_value) {
    print "not ok $test\n";
 }
 
+
+# Test the setting of a blessed object from the cache
+
+$test++;
+
+my $key2 = 'key2';
+
+$status = $cache1->set($key2, $cache1);
+
+if ($status == $sSUCCESS) {
+    print "ok $test\n";
+} else {
+   print "not ok $test\n";
+}
+
+# Test the getting of a blessed object from the cache
+
+$test++;
+
+my $cache1_retrieved = $cache1->get($key2);
+
+$val1_retrieved = $cache1_retrieved->get($key);
+
+if ($val1_retrieved eq $seed_value) {
+    print "ok $test\n";
+} else {
+   print "not ok $test\n";
+}
+
+
+
 # Test the getting of the scalar from a subprocess
+#
+# Note, this uses an explicit path the perl to avoid a taint.  
+# Not good, right?
 
-$test = 5;
+$test++;
 
-$status = system("perl", "-Iblib/lib", "./test/test_get.pl", 
-		 $sTEST_CACHE_KEY, $sTEST_NAMESPACE, $sTEST_USERNAME, $sTEST_CACHE_DEPTH, 
-		 $key, $seed_value);
+$status = system("/usr/bin/perl", "-Iblib/lib", "./test/test_get.pl", 
+		 $sTEST_CACHE_KEY, $sTEST_NAMESPACE, $sTEST_USERNAME, 
+		 $sTEST_CACHE_DEPTH, $implementation, $key, "\"$seed_value\"");
 
 if ($status == 0) {
     print "ok $test\n";
 } else {
-    print "not okay $test\n";
+    print "not ok $test\n";
 }
 
 
 # Test checking the memory consumption of the cache
 
-$test = 6;
+$test++;
 
 my $size = File::Cache::SIZE($sTEST_CACHE_KEY);
 
 if ($size > 0) {
    print "ok $test\n";
 } else {
-    print "not okay $test\n";
+    print "not ok $test\n";
 }
 
 
 # Test clearing the cache's namespace
 
-$test = 7;
+$test++;
 
 $status = $cache1->clear();
 
@@ -118,7 +207,7 @@ if ($status == $sSUCCESS) {
 # Test the max_size limit
 # Intentionally add more data to the cache than fits in max_size
 
-$test = 8;
+$test++;
 
 my $string = 'abcdefghij';
 
@@ -153,7 +242,7 @@ if ($cache1->size > $sMAX_SIZE) {
 
 # Test the getting of a scalar after the clearing of a cache
 
-$test = 9;
+$test++;
 
 my $val2_retrieved = $cache1->get($key);
 
@@ -166,7 +255,7 @@ if ($val2_retrieved) {
 
 # Test the setting of a scalar in the cache with a immediate timeout
 
-$test = 10;
+$test++;
 
 $status = $cache1->set($key, $seed_value, 0);
 
@@ -179,7 +268,7 @@ if ($status == $sSUCCESS) {
 
 # Test the getting of a scalar from the cache that should have timed out immediately
 
-$test = 11;
+$test++;
 
 my $val3_retrieved = $cache1->get($key);
 
@@ -192,7 +281,7 @@ if ($val3_retrieved) {
 
 # Test the getting of the expired scalar using get_stale
 
-$test = 12;
+$test++;
 
 my $val3_stale_retrieved = $cache1->get_stale($key);
 
@@ -206,7 +295,7 @@ if ($val3_stale_retrieved) {
 
 # Test the setting of a scalar in the cache with a timeout in the near future
 
-$test = 13;
+$test++;
 
 $status = $cache1->set($key, $seed_value, 2);
 
@@ -219,7 +308,7 @@ if ($status == $sSUCCESS) {
 
 # Test the getting of a scalar from the cache that should not have timed out yet (unless the system is *really* slow)
 
-$test = 14;
+$test++;
 
 my $val4_retrieved = $cache1->get($key);
 
@@ -232,7 +321,7 @@ if ($val4_retrieved eq $seed_value) {
 
 # Test the getting of a scalar from the cache that should have timed out
 
-$test = 15;
+$test++;
 
 sleep(3);
 
@@ -247,7 +336,7 @@ if ($val5_retrieved) {
 
 # Test purging the cache's namespace
 
-$test = 16;
+$test++;
 
 $status = $cache1->purge();
 
@@ -259,7 +348,7 @@ if ($status == $sSUCCESS) {
 
 # Test getting the creation time of the cache entry
 
-$test = 17;
+$test++;
 
 my $timed_key = 'timed key';
 
@@ -295,7 +384,7 @@ if ($status) {
 
 # Test getting the expiration time of the cache entry
 
-$test = 18;
+$test++;
 
 my $expected_expiration_time = $cache1->get_creation_time($timed_key) + $expires_in;
 
@@ -313,7 +402,7 @@ if ($status) {
 
 # Test PURGING of a cache object
 
-$test = 19;
+$test++;
 
 $status = File::Cache::PURGE($sTEST_CACHE_KEY);
 
@@ -326,7 +415,7 @@ if ($status == $sSUCCESS) {
 
 # Test the removal of a cached file
 
-$test = 20;
+$test++;
 
 $status = $sSUCCESS;
 
@@ -353,10 +442,9 @@ if ($status == $sSUCCESS) {
 }
 
 
-
 # Test CLEARING of a cache object
 
-$test = 21;
+$test++;
 
 $status = File::Cache::CLEAR($sTEST_CACHE_KEY);
 
@@ -367,6 +455,80 @@ if ($status == $sSUCCESS) {
 }
 
 
+# Test directories not created unless needed
+
+$test++;
+
+File::Cache::CLEAR($sTEST_CACHE_KEY);
+
+if (-e $sTEST_CACHE_KEY)
+{
+   print "not ok $test\n";
+}
+
+$cache1 = new File::Cache( { cache_key => $sTEST_CACHE_KEY,
+              implementation => $implementation,
+				namespace => $sTEST_NAMESPACE } );
+
+opendir(DIR, $sTEST_CACHE_KEY) or
+  croak("Couldn't open directory $sTEST_CACHE_KEY: $!");
+          
+my @dirents = readdir(DIR);
+
+closedir DIR;
+
+my @files = grep { $_ !~ /^(\.|\.\.|.description)$/ } @dirents;
+
+if (!@files) {
+    print "ok $test\n";
+} else {
+   print "not ok $test\n";
+}
+
+File::Cache::CLEAR($sTEST_CACHE_KEY);
+
+# Test the setting of a binary scalar in the cache
+
+$test++;
+
+$cache1 = new File::Cache( { cache_key => $sTEST_CACHE_KEY,
+              implementation => $implementation,
+				namespace => $sTEST_NAMESPACE } );
+
+# Make a string of all possible ASCII characters
+$seed_value = '';
+
+for (my $i = 0; $i < 256 ; $i++)
+{
+  $seed_value .= chr($i);
+}
+
+my $binary_key = 'key1';
+
+$status = $cache1->set($binary_key, $seed_value);
+
+if ($status == $sSUCCESS) {
+    print "ok $test\n";
+} else {
+   print "not ok $test\n";
+}
+
+# Test the getting of a binary scalar from the cache
+
+$test++;
+
+my $val6_retrieved = $cache1->get($binary_key);
+
+if ($val6_retrieved eq $seed_value) {
+    print "ok $test\n";
+} else {
+   print "not ok $test\n";
+}
+
+File::Cache::CLEAR($sTEST_CACHE_KEY);
+
+}
+
 1;
 
 
