@@ -3,7 +3,7 @@
 use strict;
 use File::Cache;
 
-my $sUSAGE = "Usage: test_get.pl cache_key namespace username cache_depth key expected_value";
+my $sUSAGE = "Usage: test_get.pl cache_key namespace username cache_depth implementation key expected_value";
 
 my $cache_key = $ARGV[0] or
     die("$sUSAGE\n");
@@ -17,15 +17,24 @@ my $username = $ARGV[2] or
 my $cache_depth = $ARGV[3] or
     die("sUSAGE\n");
 
-my $key = $ARGV[4] or
+my $implementation = $ARGV[4] or
     die("sUSAGE\n");
 
-my $expected_value = $ARGV[5] or
+my $key = $ARGV[5] or
     die("sUSAGE\n");
 
+my $expected_value = $ARGV[6] or
+    die("sUSAGE\n");
+
+# strip quotes, just in case the shell didn't
+
+$expected_value =~ s|\"?([^\"]*)\"?|$1|;
+
+
 my $cache = new File::Cache( { cache_key => $cache_key, 
 			       namespace => $namespace, 
 			       username => $username, 
+			       implementation => $implementation, 
 			       cache_depth => $cache_depth } ) or
     die("Couldn't create cache");
 
